@@ -1,11 +1,11 @@
 #  File R/summary.statistics.egodata.R in package ergm.ego, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2015-2018 Statnet Commons
+#  Copyright 2015-2019 Statnet Commons
 #######################################################################
 
 
@@ -67,7 +67,7 @@
 summary_formula.egodata <- function(object,..., basis=NULL, individual=FALSE, scaleto=NULL){
   egodata <-
     if(!is.null(basis)) basis
-    else get(as.character(object[[2]]), envir=environment(object))
+    else eval_lhs.formula(object)
 
   scaling.stats <- NULL
   scaling.pos <- c(0)
