@@ -1,11 +1,11 @@
 #  File tests/testthat/test-degreedist.R in package ergm.ego, part of the Statnet suite
-#  of packages for network analysis, http://statnet.org .
+#  of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  http://statnet.org/attribution
+#  https://statnet.org/attribution
 #
-#  Copyright 2015-2018 Statnet Commons
+#  Copyright 2015-2019 Statnet Commons
 #######################################################################
 context("test-degreedist.R")
 
@@ -30,3 +30,12 @@ test_that("weighted degreedist by attribute", {
   expect_equivalent(unclass(degreedist(e, plot=FALSE, by="x")), rbind(c(2/3,1/3),
                                                                       c(1/3,2/3)))
 })
+
+test_that("weighted degreedist with weights disabled", {
+  expect_equivalent(unclass(degreedist(e, plot=FALSE, weight=FALSE)), c(1/2,1/2))
+})
+
+test_that("weighted degreedist by attribute with weights disabled", {
+  expect_equivalent(unclass(degreedist(e, plot=FALSE, by="x", weight=FALSE)), rbind(c(1/2,1/2),
+                                                                                    c(1/2,1/2)))
+})
